//
//	MiscMailApp.m -- an API to NeXT's Mail.app
//		Modified by Don Yacktman (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//

#import <misckit/MiscMailApp.h>
#import <misckit/MiscDictionary.h>
#import <appkit/appkit.h>

@interface MiscMailApp(private)
- _MiscRealInitForHostNamed:host;
@end

@implementation MiscMailApp

+ localMailer
{
	return [self mailerForHostNamed:nil];
}

+ mailerForHostNamed:aHost
{
	static id localMailer = nil;
	static id dictionary = nil;
	id theMailer = nil;

	if (aHost && strlen([aHost stringValue])) {
		if (!dictionary) {
			dictionary = [[MiscDictionary alloc] init];
		}
		theMailer = [dictionary valueForKey:aHost];
		if (!theMailer) {
			theMailer = [[super alloc] _MiscRealInitForHostNamed:aHost];
			if (theMailer) [dictionary insertKey:aHost value:theMailer];
		}
	} else { // go local if no name given
		if (!localMailer) {
			localMailer = [[super alloc] _MiscRealInitForHostNamed:nil];
		}
		theMailer = localMailer;
	}
	return theMailer;
}

- init
{
	return [self error:"You cannot use -init on MiscMailApp.\n"];
}

+ alloc
{
	return [self error:"You cannot use +alloc on MiscMailApp.\n"];
}

#define TRY_PORT_CONNECT mailPort = NXPortFromName("MailSendDemo", \
		hostNamePointer); \
	if (PORT_NULL == mailPort)

- _MiscRealInitForHostNamed:host
{
	port_t mailPort;
	const char *hostNamePointer;

	if (_initted) return self;
	if (![super init]) return nil;
	if (host && [host length]) {
		hostName = [MiscString newWithString:[host stringValue]];
		hostNamePointer = [hostName stringValue];
	} else {
		hostName = [MiscString newWithString:"localhost"];
		hostNamePointer = NULL;
	}
	// check to be sure that Mail.app is up and running first
	TRY_PORT_CONNECT { // launch mailer (or Mail.app) if we can't find the port
		// The GLOBAL dwrite "Mailer" is first checked for a valid mailer.
		// ***** change this to use the MiscDefault stuff when it comes out.
		const char *mailer = NXGetDefaultValue([NXApp appName], "Mailer");
		mailPort = NXPortFromName((mailer ? mailer : "Mail"), hostNamePointer);
		if (PORT_NULL == mailPort) { // Couldn't launch from dwrite,
			// so default to Mail.app.
			mailPort = NXPortFromName("Mail", hostNamePointer);
		}
		if (PORT_NULL != mailPort) port_deallocate(task_self(), mailPort);
	}
	TRY_PORT_CONNECT { // bail out if we still cannot connect
		[self free];
		return nil;
	}
	theSpeaker = [MiscMailSpeaker new];
	[theSpeaker setSendPort:mailPort];
	currentWindow = MISC_NO_MAIL_WINDOW; // no current window
	noclutter = NO;
	_initted = YES; // make sure the init method is not re-entered!
	return self;
}

- free
{
	[theSpeaker free];
	[hostName free];
	return self;
}

- hostname { return hostName; }
- (BOOL)noclutter { return noclutter; }
- setNoclutter:(BOOL)flag { noclutter = flag; return self; }

- sendMailTo:to subject:subject body:body
{
	return [self sendMailTo:to subject:subject body:body cc:nil andDeliver:NO];
}

- sendMailTo:to subject:subject body:body cc:cc andDeliver:(BOOL)flag
{
	[self openWindow];
	[self setTo:to];
	[self setSubject:subject];
	[self setBody:body];
	[self setCc:cc];
	if (flag) [self deliver];
	return self;
}

- openWindow
{
	if (noclutter && (currentWindow >= 0)) [self deliver];
	[theSpeaker openSend:&currentWindow];
	return self;
}

- deliver
{
	if (currentWindow >= 0) [theSpeaker deliver:currentWindow];
	currentWindow = MISC_NO_MAIL_WINDOW; // there is no longer a current window
	return self;
}


#define MESSAGE(x) 	if (sender && (currentWindow >= 0)) \
		if (strlen([sender stringValue])) \
			[theSpeaker x:[sender stringValue] inWindow:currentWindow]; \
	return self

- setSubject:sender	{ MESSAGE(setSubject);	}
- setBody:sender	{ MESSAGE(setBody);		}
- setTo:sender		{ MESSAGE(setTo);		}
- setCc:sender		{ MESSAGE(setCc);		}

// to allow target/action for everything...
- openWindow:sender { return [self openWindow]; }
- deliver:sender { return [self deliver]; }

@end
